@@ -25,6 +25,7 @@
 #include "enc_settings.h"
 
 #include "svt_log.h"
+#include "utility.h"
 
 #ifdef _WIN32
 #include <windows.h>
@@ -53,11 +54,14 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
         SVT_ERROR("Instance %u: Source Height must be at least 4\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-    if (config->pred_structure > 2 || config->pred_structure < 1) {
-        SVT_ERROR("Instance %u: Pred Structure must be [1 or 2]\n", channel_number + 1);
+    if (config->pred_structure > RANDOM_ACCESS || config->pred_structure < LOW_DELAY) {
+        SVT_ERROR("Instance %u: Pred Structure must be [%d (low delay) or %d (random access)]\n",
+                  channel_number + 1,
+                  LOW_DELAY,
+                  RANDOM_ACCESS);
         return_error = EB_ErrorBadParameter;
     }
-    if (config->pred_structure == 1 && config->pass > 0) {
+    if (config->pred_structure == LOW_DELAY && config->pass > 0) {
         SVT_ERROR("Instance %u: Multi-passes is not support with Low Delay mode \n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
@@ -136,7 +140,7 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
             channel_number + 1);
 
     if (config->force_key_frames &&
-        (config->rate_control_mode == SVT_AV1_RC_MODE_CBR || config->pred_structure != SVT_AV1_PRED_RANDOM_ACCESS)) {
+        (config->rate_control_mode == SVT_AV1_RC_MODE_CBR || config->pred_structure != RANDOM_ACCESS)) {
         SVT_WARN(
             "Instance %u: Force key frames is now supported for lowdelay but the force_key_frames flag"
             " does not need to be set be on. Please follow the app samples shown by the FTR_KF_ON_FLY_SAMPLE"
@@ -152,12 +156,12 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
         SVT_ERROR("Instance %u: Max Bitrate only supported with CRF mode\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-    if (config->rate_control_mode == SVT_AV1_RC_MODE_CBR && config->pred_structure == SVT_AV1_PRED_RANDOM_ACCESS) {
-        SVT_ERROR("CBR Rate control is currently not supported for SVT_AV1_PRED_RANDOM_ACCESS, use VBR mode\n");
+    if (config->rate_control_mode == SVT_AV1_RC_MODE_CBR && config->pred_structure == RANDOM_ACCESS) {
+        SVT_ERROR("CBR Rate control is currently not supported for RANDOM_ACCESS, use VBR mode\n");
         return_error = EB_ErrorBadParameter;
     }
-    if (config->rate_control_mode == SVT_AV1_RC_MODE_VBR && config->pred_structure == SVT_AV1_PRED_LOW_DELAY_B) {
-        SVT_ERROR("VBR Rate control is currently not supported for SVT_AV1_PRED_LOW_DELAY_B, use CBR mode\n");
+    if (config->rate_control_mode == SVT_AV1_RC_MODE_VBR && config->pred_structure == LOW_DELAY) {
+        SVT_ERROR("VBR Rate control is currently not supported for LOW_DELAY, use CBR mode\n");
         return_error = EB_ErrorBadParameter;
     }
     if (config->rate_control_mode == SVT_AV1_RC_MODE_CQP_OR_CRF && config->target_bit_rate != DEFAULT_TBR) {
@@ -264,6 +268,17 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
         SVT_ERROR("Instance %u: Only rate control mode 0~2 are supported for 2-pass \n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
+    // Stats file is checked by the app in handle_stats_file, but must be re-checked here since ffmpeg calls
+    // the library but not the app
+    if (config->rate_control_mode == SVT_AV1_RC_MODE_VBR && config->pass == ENC_SECOND_PASS) {
+        if (!config->rc_stats_buffer.buf) {
+            SVT_ERROR("Instance %u: RC stats buffer not available \n", channel_number + 1);
+            return_error = EB_ErrorBadParameter;
+        } else if (config->rc_stats_buffer.sz == 0) {
+            SVT_ERROR("Instance %u: RC stats buffer size is 0 \n", channel_number + 1);
+            return_error = EB_ErrorBadParameter;
+        }
+    }
     if (config->profile > 2) {
         SVT_ERROR("Instance %u: The maximum allowed profile value is 2 \n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
@@ -520,7 +535,7 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
         return_error = EB_ErrorBadParameter;
     }
     if (config->tune == 2) {
-        if (config->rate_control_mode != 0 || config->pred_structure != SVT_AV1_PRED_RANDOM_ACCESS) {
+        if (config->rate_control_mode != 0 || config->pred_structure != RANDOM_ACCESS) {
             SVT_ERROR("Instance %u: tune SSIM only supports CRF rate control mode currently\n",
                       channel_number + 1,
                       config->tune);
@@ -628,13 +643,13 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
             channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-    // Limit 8K & 16K configurations ( due to  memory constraints)
-    if ((uint64_t)(scs->max_input_luma_width * scs->max_input_luma_height) > INPUT_SIZE_4K_TH &&
+    // Block the use of M4 or lower for resolutions higher than 4K, unless still-image coding is used (due to memory constraints)
+    if (!scs->static_config.avif &&
+        (uint64_t)(scs->max_input_luma_width * scs->max_input_luma_height) > INPUT_SIZE_4K_TH &&
         config->enc_mode <= ENC_M4) {
         SVT_ERROR("Instance %u: 8k+ resolution support is limited to M5 and faster presets.\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-
     if (config->pass > 0 && scs->static_config.enable_overlays) {
         SVT_ERROR(
             "Instance %u: The overlay frames feature is currently not supported with multi-pass "
@@ -668,14 +683,6 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
         SVT_ERROR("Error instance %u: switch frame interval must be >= 0\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-    if (config->sframe_dist > 0 && config->pred_structure != SVT_AV1_PRED_LOW_DELAY_P &&
-        config->pred_structure != SVT_AV1_PRED_LOW_DELAY_B) {
-        SVT_ERROR(
-            "Error instance %u: switch frame feature only supports low delay prediction "
-            "structure\n",
-            channel_number + 1);
-        return_error = EB_ErrorBadParameter;
-    }
     if (config->sframe_dist > 0 && config->hierarchical_levels == 0) {
         SVT_ERROR("Error instance %u: switch frame feature does not support flat IPPP\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
@@ -771,7 +778,7 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
             channel_number + 1);
     }
 
-    if (config->pred_structure == 1) {
+    if (config->pred_structure == LOW_DELAY) {
         if (config->tune == 0) {
             SVT_WARN("Instance %u: Tune 0 is not applicable for low-delay, tune will be forced to 1.\n",
                      channel_number + 1);
@@ -788,6 +795,7 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
             return_error = EB_ErrorBadParameter;
         }
     }
+
     if (scs->static_config.scene_change_detection) {
         scs->static_config.scene_change_detection = 0;
         SVT_WARN(
@@ -812,14 +820,18 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
                   channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
+    if (config->enable_qm && config->min_chroma_qm_level > config->max_chroma_qm_level) {
+        SVT_ERROR("Instance %u:  Min chroma quant matrix level must not greater than max chroma quant matrix level\n",
+                  channel_number + 1);
+        return_error = EB_ErrorBadParameter;
+    }
     if (config->startup_mg_size != 0 && config->startup_mg_size != 2 && config->startup_mg_size != 3 &&
         config->startup_mg_size != 4) {
         SVT_ERROR("Instance %u: Startup MG size supported [0, 2, 3, 4]\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-
-    if (config->startup_mg_size >= config->hierarchical_levels) {
-        SVT_ERROR("Instance %u: Startup MG size must less than Hierarchical Levels\n", channel_number + 1);
+    if (config->startup_mg_size > config->hierarchical_levels) {
+        SVT_ERROR("Instance %u: Startup MG size must less than or equal to hierarchical levels\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
     if (config->startup_mg_size != 0 && config->rate_control_mode != SVT_AV1_RC_MODE_CQP_OR_CRF) {
@@ -832,12 +844,12 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
     }
 
     if (config->variance_boost_strength < 1 || config->variance_boost_strength > 4) {
-        SVT_ERROR("Instance %u: Variance boost strength must be between 1 and 4\n", channel_number + 1);
+        SVT_ERROR("Instance %u: Variance Boost strength must be between 1 and 4\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
 
     if (config->variance_octile < 1 || config->variance_octile > 8) {
-        SVT_ERROR("Instance %u: Variance boost octile must be between 1 and 8\n", channel_number + 1);
+        SVT_ERROR("Instance %u: Variance Boost octile must be between 1 and 8\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
 
@@ -847,7 +859,7 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs) {
     }
 
     if (config->variance_boost_curve > 2) {
-        SVT_ERROR("Instance %u: Variance boost curve must be between 0 and 2\n", channel_number + 1);
+        SVT_ERROR("Instance %u: Variance Boost curve must be between 0 and 2\n", channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
 
@@ -906,14 +918,14 @@ EbErrorType svt_av1_set_default_params(EbSvtAv1EncConfiguration *config_ptr) {
     config_ptr->target_bit_rate              = 2000513;
     config_ptr->max_bit_rate                 = 0;
     config_ptr->max_qp_allowed               = 63;
-    config_ptr->min_qp_allowed               = 4;
+    config_ptr->min_qp_allowed               = MIN_QP_AUTO;
     config_ptr->enable_adaptive_quantization = 2;
     config_ptr->enc_mode                     = ENC_M8;
     config_ptr->intra_period_length          = -2;
     config_ptr->multiply_keyint              = false;
     config_ptr->intra_refresh_type           = 2;
-    config_ptr->hierarchical_levels          = 0;
-    config_ptr->pred_structure               = SVT_AV1_PRED_RANDOM_ACCESS;
+    config_ptr->hierarchical_levels          = HIERARCHICAL_LEVELS_AUTO;
+    config_ptr->pred_structure               = RANDOM_ACCESS;
     config_ptr->enable_dlf_flag              = true;
     config_ptr->cdef_level                   = DEFAULT;
     config_ptr->enable_restoration_filtering = DEFAULT;
@@ -921,6 +933,7 @@ EbErrorType svt_av1_set_default_params(EbSvtAv1EncConfiguration *config_ptr) {
     config_ptr->enable_dg                    = 1;
     config_ptr->fast_decode                  = 0;
     config_ptr->encoder_color_format         = EB_YUV420;
+    config_ptr->rtc                          = 0;
     // Rate control options
     // Set the default value toward more flexible rate allocation
     config_ptr->vbr_min_section_pct      = 0;
@@ -989,9 +1002,11 @@ EbErrorType svt_av1_set_default_params(EbSvtAv1EncConfiguration *config_ptr) {
     config_ptr->force_key_frames = 0;
 
     // Quant Matrices (QM)
-    config_ptr->enable_qm    = 0;
-    config_ptr->min_qm_level = 8;
-    config_ptr->max_qm_level = 15;
+    config_ptr->enable_qm           = 0;
+    config_ptr->min_qm_level        = 8;
+    config_ptr->max_qm_level        = 15;
+    config_ptr->min_chroma_qm_level = 8;
+    config_ptr->max_chroma_qm_level = 15;
 
     config_ptr->startup_mg_size                   = 0;
     config_ptr->startup_qp_offset                 = 0;
@@ -1012,6 +1027,7 @@ EbErrorType svt_av1_set_default_params(EbSvtAv1EncConfiguration *config_ptr) {
     config_ptr->avif                              = false;
     return return_error;
 }
+
 static const char *tier_to_str(unsigned in) {
     if (!in)
         return "(auto)";
@@ -1027,14 +1043,13 @@ static const char *level_to_str(unsigned in) {
     return ret;
 }
 
-//#define DEBUG_BUFFERS
 void svt_av1_print_lib_params(SequenceControlSet *scs) {
     EbSvtAv1EncConfiguration *config = &scs->static_config;
 
     SVT_INFO("-------------------------------------------\n");
-    if (config->pass == ENC_FIRST_PASS)
+    if (config->pass == ENC_FIRST_PASS) {
         SVT_INFO("SVT [config]: preset \t\t\t\t\t\t\t: Pass 1\n");
-    else {
+    } else {
         SVT_INFO("SVT [config]: %s\ttier %s\tlevel %s\n",
                  config->profile == MAIN_PROFILE               ? "main profile"
                      : config->profile == HIGH_PROFILE         ? "high profile"
@@ -1064,9 +1079,9 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
                  config->tune == 0       ? "VQ"
                      : config->tune == 1 ? "PSNR"
                                          : "SSIM",
-                 config->pred_structure == 1       ? "low delay"
-                     : config->pred_structure == 2 ? "random access"
-                                                   : "Unknown pred structure");
+                 config->pred_structure == LOW_DELAY           ? "low delay"
+                     : config->pred_structure == RANDOM_ACCESS ? "random access"
+                                                               : "Unknown pred structure");
         SVT_INFO(
             "SVT [config]: gop size / mini-gop size / key-frame type \t\t\t: "
             "%d / %d / %s\n",
@@ -1080,7 +1095,7 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
         } else {
             switch (config->rate_control_mode) {
             case SVT_AV1_RC_MODE_CQP_OR_CRF:
-                if (config->max_bit_rate)
+                if (config->max_bit_rate) {
                     SVT_INFO(
                         "SVT [config]: BRC mode / %s / max bitrate (kbps)\t\t\t: %s / %d / "
                         "%d\n",
@@ -1088,11 +1103,12 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
                         scs->tpl || scs->static_config.enable_variance_boost ? "capped CRF" : "CQP",
                         scs->static_config.qp,
                         (int)config->max_bit_rate / 1000);
-                else
+                } else {
                     SVT_INFO("SVT [config]: BRC mode / %s \t\t\t\t\t: %s / %d \n",
                              scs->tpl || scs->static_config.enable_variance_boost ? "rate factor" : "CQP Assignment",
                              scs->tpl || scs->static_config.enable_variance_boost ? "CRF" : "CQP",
                              scs->static_config.qp);
+                }
                 break;
             case SVT_AV1_RC_MODE_VBR:
                 SVT_INFO("SVT [config]: BRC mode / target bitrate (kbps)\t\t\t\t: VBR / %d \n",
@@ -1108,11 +1124,11 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
         }
         if (config->rate_control_mode != SVT_AV1_RC_MODE_CBR) {
             if (!config->enable_variance_boost) {
-                SVT_INFO("SVT [config]: AQ mode / variance boost \t\t\t\t\t: %d / %d\n",
+                SVT_INFO("SVT [config]: AQ mode / Variance Boost \t\t\t\t\t: %d / %d\n",
                          config->enable_adaptive_quantization,
                          config->enable_variance_boost);
             } else {
-                SVT_INFO("SVT [config]: AQ mode / variance boost strength / octile / curve \t\t: %d / %d / %d / %d\n",
+                SVT_INFO("SVT [config]: AQ mode / Variance Boost strength / octile / curve \t\t: %d / %d / %d / %d\n",
                          config->enable_adaptive_quantization,
                          config->variance_boost_strength,
                          config->variance_octile,
@@ -1126,10 +1142,9 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
                      config->film_grain_denoise_apply,
                      config->film_grain_denoise_strength);
         }
-        SVT_INFO("SVT [config]: sharpness / luminance-based QP bias \t\t\t: %d / %d\n",
+        SVT_INFO("SVT [config]: sharpness / luminance-based QP bias \t\t\t\t: %d / %d\n",
                  config->sharpness,
                  config->luminance_qp_bias);
-        SVT_INFO("");
 
         switch (config->enable_tf) {
         case 1:
@@ -1140,7 +1155,7 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
         default: break;
         }
     }
-#ifdef DEBUG_BUFFERS
+#if DEBUG_BUFFERS
     SVT_INFO("SVT [config]: INPUT / OUTPUT \t\t\t\t\t\t\t: %d / %d\n",
              scs->input_buffer_fifo_init_count,
              scs->output_stream_buffer_fifo_init_count);
@@ -1149,26 +1164,12 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
              scs->pa_reference_picture_buffer_init_count,
              scs->reference_picture_buffer_init_count,
              scs->me_pool_init_count);
-    SVT_INFO("SVT [config]: ME_SEG_W0 / ME_SEG_W1 / ME_SEG_W2 / ME_SEG_W3 \t\t\t: %d / %d / %d / %d\n",
-             scs->me_segment_column_count_array[0],
-             scs->me_segment_column_count_array[1],
-             scs->me_segment_column_count_array[2],
-             scs->me_segment_column_count_array[3]);
-    SVT_INFO("SVT [config]: ME_SEG_H0 / ME_SEG_H1 / ME_SEG_H2 / ME_SEG_H3 \t\t\t: %d / %d / %d / %d\n",
-             scs->me_segment_row_count_array[0],
-             scs->me_segment_row_count_array[1],
-             scs->me_segment_row_count_array[2],
-             scs->me_segment_row_count_array[3]);
-    SVT_INFO("SVT [config]: ME_SEG_W0 / ME_SEG_W1 / ME_SEG_W2 / ME_SEG_W3 \t\t\t: %d / %d / %d / %d\n",
-             scs->enc_dec_segment_col_count_array[0],
-             scs->enc_dec_segment_col_count_array[1],
-             scs->enc_dec_segment_col_count_array[2],
-             scs->enc_dec_segment_col_count_array[3]);
-    SVT_INFO("SVT [config]: ME_SEG_H0 / ME_SEG_H1 / ME_SEG_H2 / ME_SEG_H3 \t\t\t: %d / %d / %d / %d\n",
-             scs->enc_dec_segment_row_count_array[0],
-             scs->enc_dec_segment_row_count_array[1],
-             scs->enc_dec_segment_row_count_array[2],
-             scs->enc_dec_segment_row_count_array[3]);
+    SVT_INFO("SVT [config]: ME_SEG_W / ME_SEG_H \t\t\t: %d / %d / %d / %d\n",
+             scs->me_segment_col_count_array,
+             scs->me_segment_row_count_array);
+    SVT_INFO("SVT [config]: ENC_DEC_SEG_W / ENC_DEC_SEG_H \t\t\t: %d / %d / %d / %d\n",
+             scs->enc_dec_segment_col_count_array,
+             scs->enc_dec_segment_row_count_array);
     SVT_INFO(
         "SVT [config]: PA_P / ME_P / SBO_P / MDC_P / ED_P / EC_P \t\t\t: %d / %d / %d / %d / %d / "
         "%d\n",
@@ -1983,6 +1984,8 @@ EB_API EbErrorType svt_av1_enc_parse_parameter(EbSvtAv1EncConfiguration *config_
         {"resize-kf-denom", &config_struct->resize_kf_denom},
         {"qm-min", &config_struct->min_qm_level},
         {"qm-max", &config_struct->max_qm_level},
+        {"chroma-qm-min", &config_struct->min_chroma_qm_level},
+        {"chroma-qm-max", &config_struct->max_chroma_qm_level},
         {"use-fixed-qindex-offsets", &config_struct->use_fixed_qindex_offsets},
         {"startup-mg-size", &config_struct->startup_mg_size},
         {"variance-boost-strength", &config_struct->variance_boost_strength},
@@ -2090,12 +2093,15 @@ EB_API EbErrorType svt_av1_enc_parse_parameter(EbSvtAv1EncConfiguration *config_
         {"enable-dlf", &config_struct->enable_dlf_flag},
         {"enable-overlays", &config_struct->enable_overlays},
         {"enable-force-key-frames", &config_struct->force_key_frames},
+#if CONFIG_ENABLE_QUANT_MATRIX
         {"enable-qm", &config_struct->enable_qm},
+#endif
         {"enable-dg", &config_struct->enable_dg},
         {"gop-constraint-rc", &config_struct->gop_constraint_rc},
         {"enable-variance-boost", &config_struct->enable_variance_boost},
         {"lossless", &config_struct->lossless},
         {"avif", &config_struct->avif},
+        {"rtc", &config_struct->rtc},
     };
     const size_t bool_opts_size = sizeof(bool_opts) / sizeof(bool_opts[0]);
 
